'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var react = require('react');
var invariant = _interopDefault(require('tiny-invariant'));

var maxHorizontalScroll = function maxHorizontalScroll(dom) {
  return dom.scrollWidth - dom.clientWidth;
};

var index = (function (domRef) {
  var _ref = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
      _ref$runScroll = _ref.runScroll,
      runScroll = _ref$runScroll === void 0 ? function (offset) {
    domRef.current.scrollLeft = offset;
  } : _ref$runScroll,
      _ref$onDragStart = _ref.onDragStart,
      onDragStart = _ref$onDragStart === void 0 ? function () {} : _ref$onDragStart,
      _ref$onDragEnd = _ref.onDragEnd,
      onDragEnd = _ref$onDragEnd === void 0 ? function () {} : _ref$onDragEnd;

  var internalState = react.useRef({
    lastMousePosition: null,
    isMouseDown: false,
    isScrolling: false
  });
  var scroll = react.useCallback(function (position) {
    !(domRef.current !== null) ? process.env.NODE_ENV !== "production" ? invariant(false, "Trying to scroll to the bottom, but no element was found.\n      Did you call this scrollBottom before the component with this hook finished mounting?") : invariant(false) : void 0;
    var offset = Math.min(maxHorizontalScroll(domRef.current), position);
    runScroll(offset);
  }, [runScroll]);
  var onMouseDown = react.useCallback(function (e) {
    internalState.current.isMouseDown = true;
    internalState.current.lastMousePosition = e.clientX;
  }, []);

  var onMouseUp = function onMouseUp() {
    internalState.current.isMouseDown = false;
    internalState.current.lastMousePosition = null;

    if (internalState.current.isScrolling) {
      internalState.current.isScrolling = false;
      onDragEnd();
    }
  };

  var onMouseMove = function onMouseMove(e) {
    if (!internalState.current.isMouseDown) {
      return;
    }

    if (!internalState.current.isScrolling) {
      internalState.current.isScrolling = true;
      onDragStart();
    } // diff is negative because we want to scroll in the opposite direction of the movement


    var diff = -(e.clientX - internalState.current.lastMousePosition);
    internalState.current.lastMousePosition = e.clientX;
    scroll(domRef.current.scrollLeft + diff);
  };

  react.useEffect(function () {
    window.addEventListener('mouseup', onMouseUp);
    window.addEventListener('mousemove', onMouseMove);
    return function () {
      window.removeEventListener('mouseup', onMouseUp);
      window.removeEventListener('mousemove', onMouseMove);
    };
  }, []);
  return {
    events: {
      onMouseDown: onMouseDown
    }
  };
});

exports.default = index;
